#!/usr/local/bin/clisp -C

(defun write-8-bit-table (charset)
  (let* ((name (symbol-name charset))
         (tmpfile "BIN256"))
    (unwind-protect
      (progn
        (with-open-file (f tmpfile :direction :output
                                   :element-type '(unsigned-byte 8))
          (dotimes (i 256) (write-byte i f))
        )
        (with-open-file (g (concatenate 'string name ".TXT") :direction :output)
          (format g "#~8TName:             ~A to Unicode~%" name)
          (format g "#~8TUnicode version:  3.0~%")
          (format g "#~8TTable version:    1.0~%")
          (format g "#~8TTable format:     Format A~%")
          (format g "#~8TDate:             ~1{~5@*~D-~4@*~2,'0D-~3@*~2,'0D~:}~%"
                    (multiple-value-list (get-decoded-time)))
          (format g "#       Generated by CLISP~%")
          (format g "#~%")
          (with-open-file (f tmpfile :direction :input :external-format charset)
            (dotimes (in 256)
              (let ((out #\uFFFD))
                (file-position f in)
                (ignore-errors (setq out (read-char f)))
                ; CLISP's line-terminator conversion is too intelligent.
                (when (and (eql in 13) (eql out #\u000A)) (setq out #\u000D))
                (unless (eql out #\uFFFD)
                  (format g "0x~2,'0X~C0x~4,'0X~C#~C~A~%"
                            in #\Tab (char-code out) #\Tab
                            #\Tab (sys::unicode-attributes out)
          ) ) ) ) )
          (format t "Created ~A~%" (pathname g))
        )
      )
      (delete-file tmpfile)
) ) )

(mapc #'write-8-bit-table '(
  charset:ASCII
  charset:HP-ROMAN8
  charset:ISO-8859-1
  charset:ISO-8859-2
  charset:ISO-8859-3
  charset:ISO-8859-4
  charset:ISO-8859-5
  charset:ISO-8859-6
  charset:ISO-8859-7
  charset:ISO-8859-8
  charset:ISO-8859-9
  charset:ISO-8859-10
  charset:ISO-8859-13
  charset:ISO-8859-14
  charset:ISO-8859-15
  charset:JIS_X0201
  charset:KOI8-R
  charset:MAC-ROMAN
  charset:MAC-CENTRAL-EUROPE
  charset:MAC-ICELAND
  charset:MAC-CROATIAN
  charset:MAC-ROMANIA
  charset:MAC-CYRILLIC
  charset:MAC-UKRAINE
  charset:MAC-GREEK
  charset:MAC-TURKISH
  charset:MAC-HEBREW
  charset:MAC-ARABIC
  charset:MAC-THAI
  charset:MAC-SYMBOL
  charset:MAC-DINGBAT
  charset:NEXTSTEP
  charset:CP1250
  charset:CP1251
  charset:CP1252
  charset:CP1253
  charset:CP1254
  charset:CP1255
  charset:CP1256
  charset:CP1257
  charset:CP1258
))
