#!/bin/sh
# Tests all glibc supported encodings.
cat <<EOF |
  # Single-byte and other "small" encodings come here.
  # Keep this list in the same order as gconv-modules.
  #
  # charset name    table name          comment
  ASCII             ANSI_X3.4-1968
  ISO646-GB         BS_4730
  ISO646-CA         CSA_Z243.4-1985-1
  ISO646-CA2        CSA_Z243.4-1985-2
  ISO646-DE         DIN_66003
  ISO646-DK         DS_2089
  ISO646-ES         ES
  ISO646-ES2        ES2
  ISO646-CN         GB_1988-80
  ISO646-IT         IT
  ISO646-JP         JIS_C6220-1969-RO
  ISO646-JP-OCR-B   JIS_C6229-1984-B
  ISO646-YU         JUS_I.B1.002
  ISO646-KR         KSC5636
  ISO646-HU         MSZ_7795.3
  ISO646-CU         NC_NC00-10
  ISO646-FR         NF_Z_62-010
  ISO646-FR1        NF_Z_62-010_1973
  ISO646-NO         NS_4551-1
  ISO646-NO2        NS_4551-2
  ISO646-PT         PT
  ISO646-PT2        PT2
  ISO646-SE         SEN_850200_B
  ISO646-SE2        SEN_850200_C
  ISO-8859-1
  ISO-8859-2
  ISO-8859-3
  ISO-8859-4
  ISO-8859-5
  ISO-8859-6
  ISO-8859-7
  ISO-8859-8
  ISO-8859-9
  ISO-8859-10
  ISO-8859-11
  ISO-8859-13
  ISO-8859-14
  ISO-8859-15
  ISO-8859-16
  T.61-8BIT
  ISO_6937
  ISO_6937-2        ISO-IR-90
  KOI-8
  KOI8-R
  LATIN-GREEK
  LATIN-GREEK-1
  HP-ROMAN8
  EBCDIC-AT-DE
  EBCDIC-AT-DE-A
  EBCDIC-CA-FR
  EBCDIC-DK-NO
  EBCDIC-DK-NO-A
  EBCDIC-ES
  EBCDIC-ES-A
  EBCDIC-ES-S
  EBCDIC-FI-SE
  EBCDIC-FI-SE-A
  EBCDIC-FR
  EBCDIC-IS-FRISS
  EBCDIC-IT
  EBCDIC-PT
  EBCDIC-UK
  EBCDIC-US
  IBM037
  IBM038
  IBM256
  IBM273
  IBM274
  IBM275
  IBM277
  IBM278
  IBM280
  IBM281
  IBM284
  IBM285
  IBM290
  IBM297
  IBM420
  IBM423
  IBM424
  IBM437
  IBM500
  IBM850
  IBM851
  IBM852
  IBM855
  IBM856
  IBM857
  IBM860
  IBM861
  IBM862
  IBM863
  IBM864
  IBM865
  IBM866
  IBM866NAV
  IBM868
  IBM869
  IBM870
  IBM871
  IBM875
  IBM880
  IBM891
  IBM903
  IBM904
  IBM905
  IBM918
  IBM922
  IBM932
  IBM943
  IBM1004
  IBM1026
  IBM1046
  IBM1047
  IBM1124
  IBM1129
  IBM1132
  IBM1133
  IBM1160
  IBM1161
  IBM1162
  IBM1163
  IBM1164
  CP1125
  CP1250
  CP1251
  CP1252
  CP1253
  CP1254
  CP1255
  CP1256
  CP1257
  CP1258
  IBM874
  CP737
  CP770
  CP771
  CP772
  CP773
  CP774
  CP775
  MACINTOSH
  IEC_P27-1
  ASMO_449
  ISO-IR-99         ANSI_X3.110-1983
  ISO-IR-139        CSN_369103
  CWI
  DEC-MCS
  ECMA-CYRILLIC
  ISO-IR-153        GOST_19768-74
  GREEK-CCITT
  GREEK7
  GREEK7-OLD
  INIS
  INIS-8
  INIS-CYRILLIC
  ISO_2033          ISO_2033-1983
  ISO_5427
  ISO_5427-EXT
  ISO_5428
  ISO_10367-BOX
  MAC-IS
  MAC-UK
  CP10007
  NATS-DANO
  NATS-SEFI
  WIN-SAMI-2        SAMI-WS2
  ISO-IR-197
  TIS-620
  KOI8-U
  ISIRI-3342
  VISCII
  KOI8-T
  GEORGIAN-PS
  GEORGIAN-ACADEMY
  ISO-IR-209
  MAC-SAMI
  ARMSCII-8
  TCVN5712-1
  TSCII
  PT154
  RK1048
  IBM1025
  IBM1122
  IBM1137
  IBM1153
  IBM1154
  IBM1155
  IBM1156
  IBM1157
  IBM1158
  IBM803
  IBM901
  IBM902
  IBM921
  IBM1008
  IBM1097
  IBM1122
  IBM1123
  IBM1130
  IBM1140
  IBM1141
  IBM1142
  IBM1143
  IBM1144
  IBM1145
  IBM1146
  IBM1147
  IBM1148
  IBM1149
  IBM1166
  IBM1167
  IBM4517
  IBM4899
  IBM4909
  IBM4971
  IBM5347
  IBM9030
  IBM9066
  IBM9448
  IBM12712
  IBM16804
  ISO_11548-1
  MIK
  BRF
  MAC-CENTRALEUROPE
  KOI8-RU
  ISO-8859-9E
  HP-ROMAN9
  HP-TURKISH8
  HP-THAI8
  HP-GREEK8
  #
  # Multibyte encodings come here
  #
  SJIS
  CP932
  SHIFT_JISX0213
  EUC-KR
  CP949
  JOHAB
  BIG5
  BIG5HKSCS
  EUC-JP
  EUC-JP-MS
  EUC-CN            GB2312
  GBK
  EUC-TW
  EUC-JISX0213
  GB18030
  #
  # Stateful encodings not testable this way
  #
  #ISO-2022-JP
  #ISO-2022-JP-2
  #ISO-2022-JP-3
  #ISO-2022-KR
  #ISO-2022-CN
  #ISO-2022-CN-EXT
  #UTF-7
  #IBM930                               table-to dumps core
  #IBM933                               table-to dumps core
  #IBM935                               both table-from and table-to dump core
  #IBM937
  #IBM939                               table-to dumps core
  #IBM1364
  #IBM1371
  #IBM1388
  #IBM1390
  #IBM1399
  #
EOF
while read charset table; do
  case $charset in \#*) continue;; esac
  echo "Testing $charset"
  ./table-from "$charset" > "$charset".TXT
  ./table-to "$charset" | sort > "$charset".INVERSE.TXT
  diff "$charset".TXT "$charset".INVERSE.TXT | grep '^[<>]' | sed -e 's,^. ,,' > "$charset".IRREVERSIBLE.TXT
done

# ISO-8859-11 has no corresponding table. (Nearly identical to TIS-620.)

# INIS-8 charmaps table and iconv converter are severely broken.

# ISO_6937-2 charmap and converter have broken handling of combining marks.
# 0xd6..0xdb,0xe5 missing

# ISO_5428 charmap and converter have broken handling of combining marks.
# 0x21..0x27 map to private area.

# ISIRI-3342 has some irreversible mappings.

# IBM930, IBM933, IBM935, IBM937, IBM939 converters are totally broken.

# IBM856 and IBM1046 inverse converters are severely broken.

# Eliminate private use area in:
# ANSI_X3.110-1983
# CWI
# IBM868
# ISO_10367-BOX
# ISO_5428
# ISO_6937
# ISO_6937-2-ADD
# ISO-IR-90
# MACINTOSH
# MAC-IS
# NATS-DANO
# NATS-SEFI
# T.101-G2
# T.61-8BIT
# VIDEOTEX-SUPPL

