/**
 * Create a table from Unicode to CHARSET. Based on the java.nio package.
 * This is a good test for the CharToByte converter.
 *
 * @author Bruno Haible
 */
import java.io.*;
import java.nio.*;
import java.nio.charset.*;
public class table_to_nio {
  static String toHexString1 (int i) {
    return new String(new char[] { "0123456789ABCDEF".charAt(i) });
  }
  static String toHexString2 (int i) {
    return  toHexString1((i>>4)&0x0f)
           +toHexString1(i&0x0f);
  }
  static String toHexString4 (int i) {
    return  toHexString1((i>>12)&0x0f)
           +toHexString1((i>>8)&0x0f)
           +toHexString1((i>>4)&0x0f)
           +toHexString1(i&0x0f);
  }
  public static void main (String[] args) {
    if (args.length != 1) {
      System.err.println("Usage: java table_to_nio charset");
      System.exit(1);
    }
    String charset = args[0];
    CharsetEncoder converter = null;
    try {
      converter = Charset.forName(charset).newEncoder();
    } catch (IllegalArgumentException e) {
      System.err.println("no converter for "+charset);
      System.exit(1);
    }
    converter = converter.onMalformedInput(CodingErrorAction.REPORT);
    converter = converter.onUnmappableCharacter(CodingErrorAction.REPORT);
    for (int i = 0; i < 0x110000; i++) {
      char[] in =
        (i < 0x10000
         ? new char[] { (char)i }
         : new char[] { (char)(0xd800 + ((i - 0x10000) >> 10)),
                        (char)(0xdc00 + ((i - 0x10000) & 0x3ff)) });
      try {
        ByteBuffer out = converter.encode(CharBuffer.wrap(in));
        System.out.print("0x");
        for (int j = 0; j < out.remaining(); j++)
          System.out.print(toHexString2(out.get(j)));
        System.out.println("\t0x" + (i<0x10000 ? toHexString4(i) : Integer.toHexString(i).toUpperCase()));
      } catch (CharacterCodingException e) {
      }
    }
  }
}
